package com.hzya.frame.nifi.client;

import com.fasterxml.jackson.databind.ObjectMapper;
import com.hzya.frame.nifi.config.NifiServiceConfig;
import okhttp3.*;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.io.IOException;
import java.io.InputStream;
import java.nio.charset.StandardCharsets;
import java.util.Map;
import java.util.concurrent.atomic.AtomicReference;

/**
 * nifi客户端请求处理
 *
 * @Author：liuyang
 * @Package：com.hzya.frame.nifi.client
 * @Project：fw-nifi
 * @name：NifiClient
 * @Date：2025/5/14 10:00
 * @Filename：NifiClient
 */
@Component
public class NifiClient {

    private final NifiServiceConfig config;
    private final OkHttpClient httpClient;
    private final ObjectMapper objectMapper;
    private final AtomicReference<String> accessToken = new AtomicReference<>();
    private volatile long tokenIssueTime; // Token 发行时间（毫秒）
    private volatile long tokenExpiration; // Token 过期时间（毫秒）
    Logger logger = LoggerFactory.getLogger(NifiClient.class);

    @Autowired
    public NifiClient(NifiServiceConfig config, OkHttpClient httpClient, ObjectMapper objectMapper) {
        this.config = config;
        this.httpClient = httpClient;
        this.objectMapper = objectMapper;
        initializeToken();
    }

    private void initializeToken() {
        try {
            accessToken.set(getAccessToken());
            tokenIssueTime = System.currentTimeMillis();
            tokenExpiration = tokenIssueTime + (12 * 60 * 60 * 1000); // 默认 12 小时
            logger.info("令牌已初始化，过期时间：" + new java.util.Date(tokenExpiration));
        } catch (Exception e) {
            throw new RuntimeException("初始化NiFi访问令牌失败： " + e.getMessage());
        }
    }

    public <T> T get(String path, Class<T> responseType) throws IOException {
        return executeRequestWithRetry(() -> {
            Request request = new Request.Builder().url(config.getApiUrl() + path).get().header("Authorization", "Bearer " + accessToken.get()).build();
            return executeRequest(request, responseType);
        });
    }

    /**
     * 执行GET请求并返回文件流，供调用者处理（如保存到文件或浏览器下载）
     */
    public InputStream getAsStream(String path) throws IOException {
        return executeRequestWithRetry(() -> {
            Request request = new Request.Builder().url(config.getApiUrl() + path).get().header("Authorization", "Bearer " + accessToken.get()).build();
            return executeStreamRequest(request);
        });
    }

    public <T> T post(String path, Object requestBody, Class<T> responseType) throws IOException {
        String jsonBody = objectMapper.writeValueAsString(requestBody);
        RequestBody body = RequestBody.create(jsonBody, MediaType.get("application/json; charset=utf-8"));
        return executeRequestWithRetry(() -> {
            Request request = new Request.Builder().url(config.getApiUrl() + path).post(body).header("Authorization", "Bearer " + accessToken.get()).build();
            return executeRequest(request, responseType);
        });
    }

    public <T> T delete(String path, Class<T> responseType) throws IOException {
        return executeRequestWithRetry(() -> {
            Request request = new Request.Builder().url(config.getApiUrl() + path).delete().header("Authorization", "Bearer " + accessToken.get()).build();
            return executeRequest(request, responseType);
        });
    }

    public <T> T delete(String path, Map<String, String> queryParams, Class<T> responseType) throws IOException {
        // 构建 URL 并添加查询参数
        HttpUrl.Builder urlBuilder = HttpUrl.parse(config.getApiUrl() + path).newBuilder();
        if (queryParams != null) {
            queryParams.forEach(urlBuilder::addQueryParameter);
        }
        String url = urlBuilder.build().toString();

        return executeRequestWithRetry(() -> {
            Request request = new Request.Builder().url(url).delete().header("Authorization", "Bearer " + accessToken.get()).build();
            return executeRequest(request, responseType);
        });
    }

    public <T> T put(String path, Object requestBody, Class<T> responseType) throws IOException {
        String jsonBody = objectMapper.writeValueAsString(requestBody);
        RequestBody body = RequestBody.create(jsonBody, MediaType.get("application/json; charset=utf-8"));
        return executeRequestWithRetry(() -> {
            Request request = new Request.Builder().url(config.getApiUrl() + path).put(body).header("Authorization", "Bearer " + accessToken.get()).build();
            return executeRequest(request, responseType);
        });
    }

    private <T> T executeRequestWithRetry(IOExceptionRunnable<T> runnable) throws IOException {
        int maxRetries = 2;
        for (int attempt = 0; attempt < maxRetries; attempt++) {
            try {
                checkAndRefreshToken();
                return runnable.run();
            } catch (IOException e) {
                if (attempt == maxRetries - 1 || !isTokenExpiredError(e)) {
                    throw e;
                }
                logger.info("令牌可能已过期，请刷新并重试...");
                try {
                    accessToken.set(getAccessToken());
                    tokenIssueTime = System.currentTimeMillis();
                    tokenExpiration = tokenIssueTime + (12 * 60 * 60 * 1000); // 默认 12 小时
                } catch (Exception ex) {
                    throw new IOException("刷新令牌失败： " + ex.getMessage(), e);
                }
            }
        }
        throw new IOException("已达到最大重试次数");
    }

    private void checkAndRefreshToken() throws IOException {
        long currentTime = System.currentTimeMillis();
        if (currentTime > tokenExpiration - 300000) { // 提前 5 分钟刷新
            try {
                accessToken.set(getAccessToken());
                tokenIssueTime = System.currentTimeMillis();
                tokenExpiration = tokenIssueTime + (12 * 60 * 60 * 1000); // 默认 12 小时
                logger.info("令牌已刷新，到期时间： " + new java.util.Date(tokenExpiration));
            } catch (Exception e) {
                throw new IOException("刷新令牌失败： " + e.getMessage());
            }
        }
    }

    private boolean isTokenExpiredError(IOException e) {
        // 判断是否为 401 错误（Token 过期）
        return e.getMessage().contains("401") || e.getMessage().contains("Unauthorized");
    }

    private <T> T executeRequest(Request request, Class<T> responseType) throws IOException {
        try (Response response = httpClient.newCall(request).execute()) {
            String responseBody = response.body().string();
            logger.info("响应体：" + responseBody);
            if (!response.isSuccessful()) {
                throw new IOException("意外的响应码: " + responseBody);
            }
            return objectMapper.readValue(responseBody, responseType);
        }
    }

    public synchronized String getAccessToken() throws Exception {
        // 创建信任所有证书的 OkHttpClient
//        OkHttpClient client = httpClient.newBuilder().sslSocketFactory(createTrustAllSslSocketFactory(), createTrustAllTrustManager()).hostnameVerifier((hostname, session) -> true).build();

        MediaType mediaType = MediaType.parse("application/x-www-form-urlencoded");
        RequestBody body = RequestBody.create(mediaType, "username=" + config.getUsername() + "&password=" + config.getPassword());
        Request request = new Request.Builder().url(config.getApiUrl() + "/access/token").post(body).addHeader("Content-Type", "application/x-www-form-urlencoded").addHeader("Accept", "*/*").addHeader("User-Agent", "fw-nifi-client/1.0").addHeader("Connection", "keep-alive").build();

        try (Response response = httpClient.newCall(request).execute()) {
            if (!response.isSuccessful()) {
                throw new IOException("获取访问令牌失败，响应码: " + response.code() + " - " + response.message());
            }
            byte[] bytes = response.body().bytes();
            String newToken = new String(bytes, StandardCharsets.UTF_8);
            return newToken;
        }
    }

//    private SSLSocketFactory createTrustAllSslSocketFactory() throws Exception {
//        SSLContext sslContext = SSLContext.getInstance("SSL");
//        sslContext.init(null, new TrustManager[]{createTrustAllTrustManager()}, new SecureRandom());
//        return sslContext.getSocketFactory();
//    }

//    private X509TrustManager createTrustAllTrustManager() {
//        return new X509TrustManager() {
//            @Override
//            public void checkClientTrusted(X509Certificate[] chain, String authType) {
//            }
//
//            @Override
//            public void checkServerTrusted(X509Certificate[] chain, String authType) {
//            }
//
//            @Override
//            public X509Certificate[] getAcceptedIssuers() {
//                return new X509Certificate[0];
//            }
//        };
//    }

    private InputStream executeStreamRequest(Request request) throws IOException {
        Response response = httpClient.newCall(request).execute();
        if (!response.isSuccessful()) {
            String responseBody = response.body().string();
            response.close();
            throw new IOException("意外的响应码: " + responseBody);
        }
        return response.body().byteStream();
    }

    // 功能接口，用于重试逻辑
    @FunctionalInterface
    private interface IOExceptionRunnable<T> {
        T run() throws IOException;
    }
}